// Generated by purs version 0.12.0
"use strict";
var $foreign = require("./foreign.js");
var Control_Semigroupoid = require("../Control.Semigroupoid/index.js");
var Data_Function = require("../Data.Function/index.js");
var Data_Unit = require("../Data.Unit/index.js");

// | A `Functor` is a type constructor which supports a mapping operation
// | `map`.
// |
// | `map` can be used to turn functions `a -> b` into functions
// | `f a -> f b` whose argument and return types use the type constructor `f`
// | to represent some computational context.
// |
// | Instances must satisfy the following laws:
// |
// | - Identity: `map identity = identity`
// | - Composition: `map (f <<< g) = map f <<< map g`
var Functor = function (map) {
    this.map = map;
};
var map = function (dict) {
    return dict.map;
};

// | `mapFlipped` is `map` with its arguments reversed. For example:
// |
// | ```purescript
// | [1, 2, 3] <#> \n -> n * n
// | ```
var mapFlipped = function (dictFunctor) {
    return function (fa) {
        return function (f) {
            return map(dictFunctor)(f)(fa);
        };
    };
};

// | The `void` function is used to ignore the type wrapped by a
// | [`Functor`](#functor), replacing it with `Unit` and keeping only the type
// | information provided by the type constructor itself.
// |
// | `void` is often useful when using `do` notation to change the return type
// | of a monadic computation:
// |
// | ```purescript
// | main = forE 1 10 \n -> void do
// |   print n
// |   print (n * n)
// | ```
var $$void = function (dictFunctor) {
    return map(dictFunctor)(Data_Function["const"](Data_Unit.unit));
};

// | A version of `voidRight` with its arguments flipped.
var voidLeft = function (dictFunctor) {
    return function (f) {
        return function (x) {
            return map(dictFunctor)(Data_Function["const"](x))(f);
        };
    };
};

// | Ignore the return value of a computation, using the specified return value
// | instead.
var voidRight = function (dictFunctor) {
    return function (x) {
        return map(dictFunctor)(Data_Function["const"](x));
    };
};
var functorFn = new Functor(Control_Semigroupoid.compose(Control_Semigroupoid.semigroupoidFn));
var functorArray = new Functor($foreign.arrayMap);

// | Apply a value in a computational context to a value in no context.
// |
// | Generalizes `flip`.
// |
// | ```purescript
// | longEnough :: String -> Bool
// | hasSymbol :: String -> Bool
// | hasDigit :: String -> Bool
// | password :: String
// |
// | validate :: String -> Array Bool
// | validate = flap [longEnough, hasSymbol, hasDigit]
// | ```
// |
// | ```purescript
// | flap (-) 3 4 == 1
// | threeve <$> Just 1 <@> 'a' <*> Just true == Just (threeve 1 'a' true)
// | ```
var flap = function (dictFunctor) {
    return function (ff) {
        return function (x) {
            return map(dictFunctor)(function (f) {
                return f(x);
            })(ff);
        };
    };
};
module.exports = {
    Functor: Functor,
    map: map,
    mapFlipped: mapFlipped,
    "void": $$void,
    voidRight: voidRight,
    voidLeft: voidLeft,
    flap: flap,
    functorFn: functorFn,
    functorArray: functorArray
};
